@@ -1,11 +1,13 @@
 """
 $Id$
 
-This file is part of the pydns project.
-Homepage: http://pydns.sourceforge.net
+This file is part of the py3dns project.
+Homepage: https://launchpad.net/py3dns
 
 This code is covered by the standard Python License. See LICENSE for details.
 
+Changes for Python3 port © 2011-14 Scott Kitterman <scott@kitterman.com>
+
     Base functionality. Request and Response classes, that sort of thing.
 """
 
@@ -47,8 +49,12 @@ defaults= { 'protocol':'udp', 'port':53, 'opcode':Opcode.QUERY,
 
 def ParseResolvConf(resolv_path="/etc/resolv.conf"):
     "parses the /etc/resolv.conf file and sets defaults for name servers"
+    with open(resolv_path, 'r') as stream:
+        return ParseResolvConfFromIterable(stream)
+
+def ParseResolvConfFromIterable(lines):
+    "parses a resolv.conf formatted stream and sets defaults for name servers"
     global defaults
-    lines=open(resolv_path).readlines()
     for line in lines:
         line = line.strip()
         if not line or line[0]==';' or line[0]=='#':
@@ -92,6 +98,9 @@ class DnsRequest:
         self.argparse(name,args)
         self.defaults = self.args
         self.tid = 0
+        self.resulttype = ''
+        if len(self.defaults['server']) == 0:
+            raise DNSError('No working name servers discovered')
 
     def argparse(self,name,args):
         if not name and 'name' in self.defaults:
@@ -160,7 +169,18 @@ class DnsRequest:
 
     def processReply(self):
         self.args['elapsed']=(self.time_finish-self.time_start)*1000
-        u = Lib.Munpacker(self.reply)
+        if not self.resulttype:
+            u = Lib.Munpacker(self.reply)
+        elif self.resulttype == 'default':
+            u = Lib.MunpackerDefault(self.reply)
+        elif self.resulttype == 'binary':
+            u = Lib.MunpackerBinary(self.reply)
+        elif self.resulttype == 'text':
+            u = Lib.MunpackerText(self.reply)
+        elif self.resulttype == 'integer':
+            u = Lib.MunpackerInteger(self.reply)
+        else:
+            raise SyntaxError('Unknown resulttype: ' + self.resulttype)
         r=Lib.DnsResult(u,self.args)
         r.args=self.args
         #self.args=None  # mark this DnsRequest object as used.
@@ -193,17 +213,82 @@ class DnsRequest:
                 break
             except socket.error as msg: 
                 # Error 98, 'Address already in use'
-                if msg[0] != 98: raise
+                if msg.errno != 98: raise
 
     def conn(self):
         self.getSource()
         self.s.connect((self.ns,self.port))
 
+    def qry(self,*name,**args):
+        '''
+        Request function for the DnsRequest class.  In addition to standard
+        DNS args, the special pydns arg 'resulttype' can optionally be passed.
+        Valid resulttypes are 'default', 'text', 'decimal', and 'binary'.
+
+        Defaults are configured to be compatible with pydns:
+        AAAA: decimal
+        Others: text
+        '''
+        " needs a refactoring "
+        self.argparse(name,args)
+        #if not self.args:
+        #    raise ArgumentError, 'reinitialize request before reuse'
+        protocol = self.args['protocol']
+        self.port = self.args['port']
+        self.tid = random.randint(0,65535)
+        self.timeout = self.args['timeout'];
+        opcode = self.args['opcode']
+        rd = self.args['rd']
+        server=self.args['server']
+        if 'resulttype' in self.args:
+            self.resulttype = self.args['resulttype']
+        else:
+            self.resulttype = 'default'
+        if type(self.args['qtype']) == bytes or type(self.args['qtype']) == str:
+            try:
+                qtype = getattr(Type, str(self.args['qtype'].upper()))
+            except AttributeError:
+                raise ArgumentError('unknown query type')
+        else:
+            qtype = self.args['qtype']
+        if 'name' not in self.args:
+            print((self.args))
+            raise ArgumentError('nothing to lookup')
+        qname = self.args['name']
+        if qtype == Type.AXFR and protocol != 'tcp':
+            print('Query type AXFR, protocol forced to TCP')
+            protocol = 'tcp'
+        #print('QTYPE %d(%s)' % (qtype, Type.typestr(qtype)))
+        m = Lib.Mpacker()
+        # jesus. keywords and default args would be good. TODO.
+        m.addHeader(self.tid,
+              0, opcode, 0, 0, rd, 0, 0, 0,
+              1, 0, 0, 0)
+        m.addQuestion(qname, qtype, Class.IN)
+        self.request = m.getbuf()
+        try:
+            if protocol == 'udp':
+                self.sendUDPRequest(server)
+            else:
+                self.sendTCPRequest(server)
+        except socket.error as reason:
+            raise SocketError(reason)
+        if self.async:
+            return None
+        else:
+            return self.response
+
     def req(self,*name,**args):
         " needs a refactoring "
         self.argparse(name,args)
         #if not self.args:
         #    raise ArgumentError, 'reinitialize request before reuse'
+        try:
+            if self.args['resulttype']:
+                raise ArgumentError('Restulttype {0} set with DNS.req, use DNS.qry to specify result type.'(format(self.args['resulttype'])))
+        except:
+            # resulttype isn't set and that's what we want for DNS.req
+            pass
         protocol = self.args['protocol']
         self.port = self.args['port']
         self.tid = random.randint(0,65535)
@@ -392,114 +477,3 @@ def ParseOSXSysConfig():
     for currentset in sets:
         defaults['server'].extend(currentset)
 
-#
-# $Log$
-# Revision 1.12.2.11.2.4  2011/08/04 22:53:09  customdesigned
-# Add OSX support
-#
-# Revision 1.12.2.11.2.3  2011/05/02 16:04:32  customdesigned
-# Don't complain about AXFR protocol unless actually changing it.
-# Reported by Ewoud Kohl van Wijngaarden.
-#
-# Revision 1.12.2.11.2.2  2011/03/23 01:42:07  customdesigned
-# Changes from 2.3 branch
-#
-# Revision 1.12.2.11.2.1  2011/02/18 19:35:22  customdesigned
-# Python3 updates from Scott Kitterman
-#
-# Revision 1.12.2.10  2008/08/01 03:58:03  customdesigned
-# Don't try to close socket when never opened.
-#
-# Revision 1.12.2.9  2008/08/01 03:48:31  customdesigned
-# Fix more breakage from port randomization patch.  Support Ipv6 queries.
-#
-# Revision 1.12.2.8  2008/07/31 18:22:59  customdesigned
-# Wait until tcp response at least starts coming in.
-#
-# Revision 1.12.2.7  2008/07/28 01:27:00  customdesigned
-# Check configured port.
-#
-# Revision 1.12.2.6  2008/07/28 00:17:10  customdesigned
-# Randomize source ports.
-#
-# Revision 1.12.2.5  2008/07/24 20:10:55  customdesigned
-# Randomize tid in requests, and check in response.
-#
-# Revision 1.12.2.4  2007/05/22 20:28:31  customdesigned
-# Missing import Lib
-#
-# Revision 1.12.2.3  2007/05/22 20:25:52  customdesigned
-# Use socket.inetntoa,inetaton.
-#
-# Revision 1.12.2.2  2007/05/22 20:21:46  customdesigned
-# Trap socket error
-#
-# Revision 1.12.2.1  2007/05/22 20:19:35  customdesigned
-# Skip bogus but non-empty lines in resolv.conf
-#
-# Revision 1.12  2002/04/23 06:04:27  anthonybaxter
-# attempt to refactor the DNSRequest.req method a little. after doing a bit
-# of this, I've decided to bite the bullet and just rewrite the puppy. will
-# be checkin in some design notes, then unit tests and then writing the sod.
-#
-# Revision 1.11  2002/03/19 13:05:02  anthonybaxter
-# converted to class based exceptions (there goes the python1.4 compatibility :)
-#
-# removed a quite gross use of 'eval()'.
-#
-# Revision 1.10  2002/03/19 12:41:33  anthonybaxter
-# tabnannied and reindented everything. 4 space indent, no tabs.
-# yay.
-#
-# Revision 1.9  2002/03/19 12:26:13  anthonybaxter
-# death to leading tabs.
-#
-# Revision 1.8  2002/03/19 10:30:33  anthonybaxter
-# first round of major bits and pieces. The major stuff here (summarised
-# from my local, off-net CVS server :/ this will cause some oddities with
-# the
-#
-# tests/testPackers.py:
-#   a large slab of unit tests for the packer and unpacker code in DNS.Lib
-#
-# DNS/Lib.py:
-#   placeholder for addSRV.
-#   added 'klass' to addA, make it the same as the other A* records.
-#   made addTXT check for being passed a string, turn it into a length 1 list.
-#   explicitly check for adding a string of length > 255 (prohibited).
-#   a bunch of cleanups from a first pass with pychecker
-#   new code for pack/unpack. the bitwise stuff uses struct, for a smallish
-#     (disappointly small, actually) improvement, while addr2bin is much
-#     much faster now.
-#
-# DNS/Base.py:
-#   added DiscoverNameServers. This automatically does the right thing
-#     on unix/ win32. No idea how MacOS handles this.  *sigh*
-#     Incompatible change: Don't use ParseResolvConf on non-unix, use this
-#     function, instead!
-#   a bunch of cleanups from a first pass with pychecker
-#
-# Revision 1.5  2001/08/09 09:22:28  anthonybaxter
-# added what I hope is win32 resolver lookup support. I'll need to try
-# and figure out how to get the CVS checkout onto my windows machine to
-# make sure it works (wow, doing something other than games on the
-# windows machine :)
-#
-# Code from Wolfgang.Strobl@gmd.de
-# win32dns.py from
-# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66260
-#
-# Really, ParseResolvConf() should be renamed "FindNameServers" or
-# some such.
-#
-# Revision 1.4  2001/08/09 09:08:55  anthonybaxter
-# added identifying header to top of each file
-#
-# Revision 1.3  2001/07/19 07:20:12  anthony
-# Handle blank resolv.conf lines.
-# Patch from Bastian Kleineidam
-#
-# Revision 1.2  2001/07/19 06:57:07  anthony
-# cvs keywords added
-#
-#
